/*************************************************************************************
	NOTEMIDILIST - a utility for listing the status of Linux sound devices

	(C) 2001 by Michael Minn - michael@michaelminn.com

	See README.txt for a complete description of this utility

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

*************************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <string.h>
#include <unistd.h>
#include <sys/ioctl.h>
#include <linux/soundcard.h>

/***********************************************************************/

int list_sequencer(char *sequencer_name)
{
	int descriptor = open(sequencer_name, O_RDONLY);

	if (descriptor < 0)
		return 0;

	printf("%s\n", sequencer_name);

	int midi_ports = 0;
	if (ioctl(descriptor, SNDCTL_SEQ_NRMIDIS, &midi_ports) < 0)
		perror("Failure reading number of MIDI devices");

	if (midi_ports <= 0)
		printf("   No configured MIDI devices\n");

	struct midi_info midi_info;
	memset(&midi_info, 0, sizeof(midi_info));
	for (midi_info.device = 0; midi_info.device < midi_ports; ++midi_info.device)
		if (ioctl(descriptor, SNDCTL_MIDI_INFO, &midi_info) < 0)
			printf("   MIDI device %d: (no name given)\n", midi_info.device);
		else
			printf("   MIDI device %d: %s\n", midi_info.device, midi_info.name);

	int synthesizers = 0;
	if (ioctl(descriptor, SNDCTL_SEQ_NRSYNTHS, &synthesizers) < 0)
		perror("   Failure reading number of synthesizers");

	if (synthesizers <= 0)
		printf("   No configured synthesizers\n");

	struct synth_info synth_info;
	memset(&synth_info, 0, sizeof(synth_info));
	for (synth_info.device = 0; synth_info.device < synthesizers; ++synth_info.device)
		if (ioctl(descriptor, SNDCTL_SYNTH_INFO, &synth_info) < 0)
			printf("   Synthesizer %d: registered but not accessible with /dev/sequencer\n", synth_info.device);
		else
			printf("   Synthesizer %d: %d voices, type %02x\n", synth_info.device, synth_info.nr_voices, synth_info.synth_type);

	close(descriptor);

	return 1;
}

int list_midi_device(char *device_name)
{
	int descriptor = open(device_name, O_RDONLY);

	if (descriptor < 0)
		return 0;

	struct midi_info midi_info;
	memset(&midi_info, 0, sizeof(midi_info));
	if (ioctl(descriptor, SNDCTL_MIDI_INFO, &midi_info) < 0)
		printf("%s\n", device_name);
	else
		printf("%s (%s)\n", device_name, midi_info.name);

	close(descriptor);
	return 1;
}

int list_mixer(char *mixer_name)
{
	int descriptor = open(mixer_name, O_RDONLY);

	if (descriptor < 0)
		return 0;

	printf("%s: ", mixer_name);

	int device_mask = 0;
	if (ioctl(descriptor, SOUND_MIXER_READ_DEVMASK, &device_mask) < 0)
		perror("   Failure reading number devices");

	if (device_mask == 0)
		printf("No configured channels");

	if (device_mask & (1 << SOUND_MIXER_VOLUME))
		printf("Volume, ");

	if (device_mask & (1 << SOUND_MIXER_BASS))
		printf("Bass, ");

	if (device_mask & (1 << SOUND_MIXER_TREBLE))
		printf("Treble, ");

	if (device_mask & (1 << SOUND_MIXER_SYNTH))
		printf("Synthesizer, ");

	if (device_mask & (1 << SOUND_MIXER_PCM))
		printf("PCM, ");

	if (device_mask & (1 << SOUND_MIXER_SPEAKER))
		printf("Speaker, ");

	if (device_mask & (1 << SOUND_MIXER_LINE))
		printf("Line in, ");

	if (device_mask & (1 << SOUND_MIXER_MIC))
		printf("Microphone, ");

	if (device_mask & (1 << SOUND_MIXER_CD))
		printf("CD, ");

	if (device_mask & (1 << SOUND_MIXER_IMIX))
		printf("Recording monitor, ");

	if (device_mask & (1 << SOUND_MIXER_ALTPCM))
		printf("Alternate PCM device, ");

	if (device_mask & (1 << SOUND_MIXER_RECLEV))
		printf("Recording level, ");

	if (device_mask & (1 << SOUND_MIXER_IGAIN))
		printf("Input gain, ");

	if (device_mask & (1 << SOUND_MIXER_OGAIN))
		printf("Output gain, ");

	printf("\n");

	close(descriptor);

	return 1;
}

int list_audio_device(char *device_name)
{
	int descriptor = open(device_name, O_RDONLY);

	if (descriptor < 0)
		return 0;

	// This is nonstandard and only implemented by USBUA100
	struct midi_info midi_info;
	memset(&midi_info, 0, sizeof(midi_info));
	if (ioctl(descriptor, SNDCTL_MIDI_INFO, &midi_info) < 0)
		printf("%s: ", device_name);
	else
		printf("%s (%s): ", device_name, midi_info.name);

	int data = 8;
	if (ioctl(descriptor, SOUND_PCM_WRITE_BITS, &data) >= 0)
		printf("8-bit, ");
	data = 16;
	if (ioctl(descriptor, SOUND_PCM_WRITE_BITS, &data) >= 0)
		printf("16-bit, ");

	data = 1;
	if (ioctl(descriptor, SOUND_PCM_WRITE_CHANNELS, &data) >= 0)
		printf("Mono, ");
	data = 2;
	if (ioctl(descriptor, SOUND_PCM_WRITE_CHANNELS, &data) >= 0)
		printf("Stereo, ");
	data = 4;
	if (ioctl(descriptor, SOUND_PCM_WRITE_CHANNELS, &data) >= 0)
		printf("Quadraphonic ");

	data = 48000;
	if (ioctl(descriptor, SOUND_PCM_WRITE_RATE, &data) >= 0)
		printf("48Khz, ");
	data = 44100;
	if (ioctl(descriptor, SOUND_PCM_WRITE_RATE, &data) >= 0)
		printf("44.1Khz, ");
	data = 22050;
	if (ioctl(descriptor, SOUND_PCM_WRITE_RATE, &data) >= 0)
		printf("22Khz, ");
	data = 11025;
	if (ioctl(descriptor, SOUND_PCM_WRITE_RATE, &data) >= 0)
		printf("11Khz, ");

	printf("\n");

	close(descriptor);

	return 1;
}

int main (int argc, char **argv)
{
	printf("NOTEMIDILIST v2001.06.24 by Michael Minn\n\n");
	printf("************* Listing of Standard Linux Devices ******************\n");

	int midi_devices = 0;
	midi_devices += list_midi_device("/dev/midi00");
	midi_devices += list_midi_device("/dev/midi01");
	midi_devices += list_midi_device("/dev/midi02");
	midi_devices += list_midi_device("/dev/midi03");
	midi_devices += list_midi_device("/dev/midi04");
	if (midi_devices <= 0)
		printf("/dev/midi: no devices available\n");

	int sequencers = 0;
	sequencers += list_sequencer("/dev/sequencer");
	sequencers += list_sequencer("/dev/sequencer1");
	sequencers += list_sequencer("/dev/sequencer2");
	if (sequencers <= 0)
		printf("/dev/sequencer: no devices available\n");

	int mixers = 0;
	mixers += list_mixer("/dev/mixer");
	mixers += list_mixer("/dev/mixer1");
	mixers += list_mixer("/dev/mixer2");
	if (sequencers <= 0)
		printf("/dev/mixer: no devices available\n");

	int audio_devices = 0;
	audio_devices += list_audio_device("/dev/dsp");
	audio_devices += list_audio_device("/dev/dsp1");
	audio_devices += list_audio_device("/dev/dsp2");
	audio_devices += list_audio_device("/dev/dsp3");
	audio_devices += list_audio_device("/dev/dsp4");
	if (audio_devices <= 0)
		printf("/dev/dsp: No audio devices available\n");

	return 0;
}
